"""
compatibility replacement for our needs among the old crypt module
that was removed in 3.13 (failed to see the earlier warnings...)

we.re only using the crypt.crypt() function, so we only need to replace that
plus, we only use MD5 for hashing, so we only need to replace that
"""

import hashlib
import base64
import os

def default_algorithm(salt):
    if salt.startswith("$1$"):
        return "md5"
    elif salt.startswith("$5$"):
        return "sha256"
    elif salt.startswith("$6$"):
        return "sha512"
    else:
        return None

def crypt(password: str, salt: str = None, algorithm: str = None) -> str:
    if salt is None:
        # Generate a random 16-character salt
        salt = base64.b64encode(os.urandom(16)).decode('utf-8')[:16]
    if algorithm is None:
        algorithm = default_algorithm(salt)

    # Create the hash using the chosen algorithm
    try:
        hash_func = getattr(hashlib, algorithm)
    except AttributeError:
        raise ValueError(f"Unknown algorithm: {algorithm}")
    hashed = hash_func((salt + password).encode()).hexdigest()

    # Format the output to mimic crypt.crypt output
    return f"${algorithm}${salt}${hashed}"


def test():
    from crypt import crypt as python_crypt
    password = "mypassword"
    salt = "$1$12345678$"
    hashed1 = crypt(password, salt)
    hashed2 = python_crypt(password, salt)
    assert hashed1 == hashed2
    print("Test passed")

if __name__ == "__main__":
    test()
